/*==============================================================================

    S E N S O R S   AMBIENT LIGHT   D R I V E R

DESCRIPTION

   Implements the driver for the al3320 ALS(Light) Sensor

Copyright (c) 2015 by QUALCOMM Technologies, Incorporated.  All Rights Reserved.
Qualcomm Technologies Proprietary and Confidential
==============================================================================*/
/*==============================================================================

                      EDIT HISTORY FOR FILE

  This section contains comments describing changes made to the module.
  Notice that changes are listed in reverse chronological order.

$Header: //source/qcom/qct/core/pkg/dsps/rel/2.2/adsp_proc/Sensors/dd/qcom/src/sns_dd_als_al3320.c#3 $


***************************************************************************
* REVISON HISTORY
* VERSION   DATE(MM/DD/YY)  WHO     DESCRIPTION
* 2.0       03/20/2017      LO      Refactoring scenario of als auto gain
* 1.0       12/20/2016      LO      Initial version of driver for AL3320
*************************************************************************** 
==============================================================================*/
/*============================================================================
                                INCLUDE FILES
============================================================================*/
#include "sns_dd_als_al3320_priv.h"     

#include "sns_memmgr.h"
#include "log_codes.h"
#include "sns_debug_str.h"
#include "sns_debug_api.h"
#include "sns_log_types.h"
#include "sns_log_api.h"


sns_ddf_sensor_e sns_dd_als_sensor_types[SNS_DD_ALSPRX_NUM_SENSOR_TYPES] =
{
  SNS_DDF_SENSOR__NONE,
  SNS_DDF_SENSOR_AMBIENT
};


/*===========================================================================
FUNCTION      sns_dd_als_al3320_enable_sched_data

DESCRIPTION   enable schedule data
===========================================================================*/
sns_ddf_status_e sns_dd_als_al3320_enable_sched_data(sns_ddf_handle_t dd_handle,
                                                         sns_ddf_sensor_e sensor,
                                                         bool             enable
                                                         )
{
    sns_dd_als_state_t* state = dd_handle;
    uint8_t byteVal, count = 0;
    sns_ddf_status_e status = SNS_DDF_SUCCESS;

    DDF_MSG_2(HIGH, "al3320_enable_sched_data() start, sensor %d enable %d", sensor, enable);

    /* Input sanity check */
    if (state == NULL)
    {
        DDF_MSG_0(HIGH, "al3320_enable_sched_data() state is NULL err");
        return SNS_DDF_EINVALID_PARAM;
    }

    /* check parameter - sensor */
    if (sensor != SNS_DDF_SENSOR_AMBIENT)
    {
        DDF_MSG_1(HIGH, "al3320_enable_sched_data() sensor type err, sensor: %d", sensor);
        return SNS_DDF_EINVALID_PARAM;
    }

    /* Check if the platform supports interrupts. */
    if (!sns_ddf_signal_irq_enabled() || (state->int_gpio == 0))
    {
        DDF_MSG_1(HIGH, "al3320_enable_sched_data() it does not support interrupts err, int_gpio: %d", state->int_gpio);
        return SNS_DDF_EFAIL;
    }

    if (enable) {
        byteVal = SNS_CLR_ALL_INT;
        while ((status = sns_dd_als_al3320_write_byte(state->port_handle, AL3320_FLAG_STATUS, &byteVal)) != SNS_DDF_SUCCESS) {          
            if (count == 10) {
                DDF_MSG_2(HIGH, "al3320_enable_sched_data() clear interrupt err: %d, count: %d", status, count);
                count = 0;
                return status;
            }
            count++;            
            sns_ddf_delay(5000);
        }

        /* IRQ's are enabled, register the irq*/
        while ((status = sns_ddf_signal_register(state->int_gpio, dd_handle,
                                         &AL3320_DD_IF,                                      
                                         SNS_DDF_SIGNAL_IRQ_FALLING)) != SNS_DDF_SUCCESS) {
            if (count == 5) {
                DDF_MSG_3(HIGH, "al3320_enable_sched_data() irq(%d) registry err: %d, count: %d", state->int_gpio, status, count);
                count = 0;
                return status;
            }
            count++;
            sns_ddf_delay(5000);
        }
        if (status == SNS_DDF_SUCCESS) {
            DDF_MSG_0(HIGH, "al3320_enable_sched_data() registry success");
        }       
    } else {
        status = sns_ddf_signal_deregister(state->int_gpio);
        if (status != SNS_DDF_SUCCESS) {
            DDF_MSG_1(HIGH, "al3320_enable_sched_data() - irq de-registry err: %d", status);
        }
    }

    return status;
}

/*===========================================================================
FUNCTION      sns_dd_als_al3320_set_reg_values_from_nv

DESCRIPTION   set default parameter from nv
===========================================================================*/
void sns_dd_als_al3320_set_reg_values_from_nv(sns_dd_als_state_t* state)
{
    if(state->sns_dd_als_common_db.nv_db.ALS_thresh_high != 0)
    {
    
        state->chip.setup.als.thresh_high = state->sns_dd_als_common_db.nv_db.ALS_thresh_high;
        DDF_MSG_1(MED, "ALS High written %d", state->chip.setup.als.thresh_high);
    }

    if(state->sns_dd_als_common_db.nv_db.ALS_thresh_low != 0)
    {
    
        state->chip.setup.als.thresh_low = state->sns_dd_als_common_db.nv_db.ALS_thresh_low;
        DDF_MSG_1(MED, "ALS Low Written %d", state->chip.setup.als.thresh_low);
    }    
}

/*===========================================================================
FUNCTION      sns_dd_als_al3320_init

DESCRIPTION   init
===========================================================================*/
sns_ddf_status_e sns_dd_als_al3320_init(
                                            sns_ddf_handle_t*        dd_ptr,
                                            sns_ddf_handle_t         smgr_handle,
                                            sns_ddf_nv_params_s*     reg_params,
                                            sns_ddf_device_access_s  device_info[],
                                            uint32_t                 num_devices,
                                            sns_ddf_memhandler_s*    memhandler,
                                            sns_ddf_sensor_e*        sensors[],
                                            uint32_t*                num_sensors
                                            )
{
    sns_ddf_status_e       status = SNS_DDF_SUCCESS;
    sns_dd_als_state_t* ptr;

    DDF_MSG_0(HIGH, "AL3320 Init");

#if AL3320_ENABLE_UIMG_SUPPORT
    sns_ddf_smgr_set_uimg_refac(smgr_handle);
#endif

    if ( (dd_ptr == NULL)      ||
         (num_sensors == NULL) ||
         (sensors == NULL) )
    {
        DDF_MSG_0(HIGH, "invalid param");
        return SNS_DDF_EINVALID_PARAM;
    }

    status = sns_ddf_malloc_ex((void**)&ptr, sizeof(sns_dd_als_state_t), smgr_handle);
    if ( status != SNS_DDF_SUCCESS )
    {
        DDF_MSG_1(MED, "AL3320 Init memory alloc ptr err: %d", status);
        return SNS_DDF_ENOMEM;
    }

    status = sns_ddf_malloc_ex((void **) &ptr->s_data, 3 * sizeof(sns_ddf_sensor_data_s), smgr_handle);
    if (status != SNS_DDF_SUCCESS) {
        DDF_MSG_1(MED, "AL3320 Init memory alloc s_data err: %d", status);
        sns_ddf_mfree_ex((void *)ptr , smgr_handle);
        return SNS_DDF_ENOMEM;
    }

    status = sns_ddf_malloc_ex((void **) &ptr->samples, 3 * sizeof(sns_ddf_sensor_sample_s), smgr_handle);
    if (status != SNS_DDF_SUCCESS) {
        DDF_MSG_1(MED, "AL3320 Init memory alloc samples err: %d", status);
        sns_ddf_mfree_ex(ptr->s_data, smgr_handle);
        sns_ddf_mfree_ex((void *)ptr , smgr_handle);
        return SNS_DDF_ENOMEM;
    }

    ptr->smgr_handle    = smgr_handle;
    ptr->int_gpio       = device_info->first_gpio;
    ptr->chip.int_gpio  = ptr->int_gpio;
    ptr->als_auto_gain_delay_time = sns_ddf_convert_usec_to_tick(AL3320_ALS_AUTO_GAIN_USDELAY);
    DDF_MSG_1(HIGH, "AL3320 Init gpio %d", ptr->chip.int_gpio);

    status = sns_ddf_open_port(&ptr->port_handle, &device_info->port_config);
    if ( status != SNS_DDF_SUCCESS )
    {
        DDF_MSG_1(MED, "AL3320 Init open port_handle err: %d", status);
        sns_ddf_mfree_ex(ptr->s_data, smgr_handle);
        sns_ddf_mfree_ex(ptr->samples, smgr_handle);
        sns_ddf_mfree_ex((void *)ptr , smgr_handle);
        return SNS_DDF_EBUS;
    }

    device_info->port_config.bus_config.i2c->dev_type = SNS_DDF_I2C_DEVICE_DEFAULT;
    status = sns_ddf_open_port(&ptr->ZBW_port_handle, &device_info->port_config);
    if( status != SNS_DDF_SUCCESS )
    {
        DDF_MSG_1(MED, "AL3320 Init open ZBW_port err: %d", status);
        sns_ddf_close_port(ptr->port_handle);
        sns_ddf_mfree_ex(ptr->s_data, smgr_handle);
        sns_ddf_mfree_ex(ptr->samples, smgr_handle);
        sns_ddf_mfree_ex((void *)ptr , smgr_handle);
        return SNS_DDF_EBUS;
    }


    /* Fill out output parameters*/
    *num_sensors = SNS_DD_ALSPRX_NUM_SENSOR_TYPES;
    *sensors     = (sns_ddf_sensor_e *)sns_dd_als_sensor_types;
    *dd_ptr      = (sns_ddf_handle_t) ptr;

     DDF_MSG_0(HIGH, "past irq and timer enable");
     ptr->nv_db_size = 0;
     bool data_from_registry = false;     

    /* Recall NV parameters, if any*/
    if(reg_params)
    {
        DDF_MSG_0(HIGH, "yes registry params available");
        if ( (reg_params->status == SNS_DDF_SUCCESS) &&
             (reg_params->data_len >= sizeof(sns_dd_nv_db_type)))
        {
            DDF_MSG_0(HIGH, "registry params match nv db type");
            sns_dd_nv_db_type *nv_ptr = (sns_dd_nv_db_type *)reg_params->data;
            if ( (nv_ptr->lsc_num == LSC_MAGIC_NUM) &&
                 (nv_ptr->drv_ID          == LSC_DRIVER_ID))
            {
                /* update value from NV parameters*/
                DDF_MSG_0(HIGH, "magic number for AL3320 verified");
                data_from_registry = true;
                ptr->nv_db_size = reg_params->data_len;

                //SNS_OS_MEMCOPY(&ptr->sns_dd_als_common_db.nv_db, nv_ptr, sizeof(sns_dd_nv_db_type));
                memcpy(&ptr->sns_dd_als_common_db.nv_db, nv_ptr, sizeof(sns_dd_nv_db_type));
                DDF_MSG_0(HIGH, "in registry params past memcopy");

                sns_dd_als_al3320_set_reg_values_from_nv(ptr);
                DDF_MSG_0(HIGH, "setting defaults from nv");
            }
        }

        if (!data_from_registry)
        {
            DDF_MSG_0(HIGH, "No registry data");
            ptr->nv_db_size = REG_FIXED_NV_SIZE_128;
            ptr->sns_dd_als_common_db.nv_db.visible_ratio = 5;
            ptr->sns_dd_als_common_db.nv_db.ir_ratio = 80;
            ptr->sns_dd_als_common_db.nv_db.dc_offset = 0;
            ptr->sns_dd_als_common_db.nv_db.als_factor  = 1;
            ptr->sns_dd_als_common_db.nv_db.lsc_num = (uint32_t)LSC_MAGIC_NUM;
            ptr->sns_dd_als_common_db.nv_db.drv_ID  = (uint32_t)LSC_DRIVER_ID;
            ptr->sns_dd_als_common_db.nv_db.device = 4;
            ptr->sns_dd_als_common_db.nv_db.ALS_thresh_high = SNS_ALS_THR_HIG_F;
            ptr->sns_dd_als_common_db.nv_db.ALS_thresh_low = SNS_ALS_THR_LOW_F;

            DDF_MSG_0(HIGH, "Set defaults for registry data");
            /* Writing the same values above to the part*/
            sns_dd_als_al3320_set_reg_values_from_nv(ptr);
            DDF_MSG_0(HIGH, "Registry Updated, sending the same values sent to registry to part");

           /*  SNS_OS_MEMZERO(ptr->sns_dd_als_common_db.nv_db.reserved, sizeof(ptr->sns_dd_als_common_db.nv_db.reserved)); (no reserved space here)*/
        }

        if (ptr->sns_dd_als_common_db.nv_db.calibratePhone)
        {
           ptr->calibration_mode = true;
           DDF_MSG_0(HIGH, "setting phone calib true");
        }
        if ((reg_params) && (!data_from_registry))
        {
           DDF_MSG_0(HIGH, "before notify");
           sns_ddf_smgr_notify_event(smgr_handle, SNS_DDF_SENSOR_AMBIENT,SNS_DDF_EVENT_UPDATE_REGISTRY_GROUP);
           DDF_MSG_0(HIGH, "after notify");
        }
    }

    status = sns_dd_als_al3320_update_registers(ptr);
    if (status != SNS_DDF_SUCCESS) {
        DDF_MSG_1(HIGH, "al3320_init() update register err: %d", status);
        sns_ddf_close_port(ptr->port_handle);
        sns_ddf_close_port(ptr->ZBW_port_handle);
        sns_ddf_mfree_ex(ptr->s_data, smgr_handle);
        sns_ddf_mfree_ex(ptr->samples, smgr_handle);
        sns_ddf_mfree_ex((void *)ptr , smgr_handle);
    }

    return status;
}

/*===========================================================================
FUNCTION      sns_dd_als_al3320_query

DESCRIPTION   als query
===========================================================================*/
sns_ddf_status_e sns_dd_als_al3320_query
(
  sns_dd_als_state_t* dd_ptr,
  sns_ddf_memhandler_s*  memhandler,
  sns_ddf_attribute_e    attrib,
  void**                 value,
  uint32_t*              num_elems
)
{
  sns_ddf_power_info_s* power_ptr;
  uint16_t*             accu_ptr;
  uint32_t*             odr_ptr;

  sns_ddf_resolution_adc_s* resp;

  DDF_MSG_0(HIGH, "sns_dd_als_al3320_query begin");

  switch ( attrib )
  {
    case SNS_DDF_ATTRIB_RANGE:
    {
      sns_ddf_range_s *device_ranges;
      if( (*value = sns_ddf_memhandler_malloc_ex(memhandler,
                      sizeof(sns_ddf_range_s), dd_ptr->smgr_handle))  == NULL)
      {
          return SNS_DDF_ENOMEM;
      }
      DDF_MSG_0(HIGH, "in get_attr range");
      device_ranges = *value;
      *num_elems = 1;
      device_ranges->min = FX_FLTTOFIX_Q16(0.01);
      device_ranges->max = FX_FLTTOFIX_Q16(10000);
      break;
    }
    case SNS_DDF_ATTRIB_RESOLUTION_ADC:
    {
      if ( (resp = sns_ddf_memhandler_malloc_ex(memhandler, sizeof(sns_ddf_resolution_adc_s), dd_ptr->smgr_handle)) == NULL )
      {
        return SNS_DDF_ENOMEM;
      }
      DDF_MSG_0(HIGH, "in get_attr resolution ADC");
      *num_elems = 1;
      //SNS_PRINTF_STRING_ID_MEDIUM_1(SNS_DBG_MOD_DSPS_DD_ALSPRX,
                                    //DBG_DD_ALSPRX_GET_ATTRIB_RES_ADC,
                                    //SNS_DD_ALSPRX_ALS_BITS);
      resp->bit_len = SNS_DD_ALSPRX_ALS_BITS;
      resp->max_freq = SNS_DD_ALSPRX_ALS_FREQ;
      *value = resp;
      break;
    }

    case SNS_DDF_ATTRIB_RESOLUTION:
    {
      sns_ddf_resolution_t* resp;
      if ( (resp = sns_ddf_memhandler_malloc_ex(memhandler, sizeof(sns_ddf_resolution_t), dd_ptr->smgr_handle)) == NULL )
      {
        return SNS_DDF_ENOMEM;
      }
      DDF_MSG_0(HIGH, "in get_attr resolution");
      *num_elems = 1;

      *resp = SNS_DD_ALS_RES;
      //SNS_PRINTF_STRING_ID_MEDIUM_1(SNS_DBG_MOD_DSPS_DD_ALSPRX,
                                    //DBG_DD_ALSPRX_GET_ATTRIB_RES,
                                    //*resp);
      *value = resp;
      break;
    }

    case SNS_DDF_ATTRIB_POWER_INFO:
      if ( (power_ptr = sns_ddf_memhandler_malloc_ex(memhandler, sizeof(sns_ddf_power_info_s), dd_ptr->smgr_handle)) == NULL )
      {
        return SNS_DDF_ENOMEM;
      }
      *num_elems = 1;
      power_ptr->active_current   = SNS_DD_ALS_PWR;
      power_ptr->lowpower_current = SNS_DD_ALSPRX_LO_PWR;
      //SNS_PRINTF_STRING_ID_MEDIUM_1(SNS_DBG_MOD_DSPS_DD_ALSPRX,
                                    //DBG_DD_ALSPRX_GET_ATTRIB_PWR,
                                    //power_ptr->active_current); // add power_ptr->lowpower_current to F3
      *(sns_ddf_power_info_s **)value = power_ptr;
      break;

    case SNS_DDF_ATTRIB_ACCURACY:
      if ( (accu_ptr = sns_ddf_memhandler_malloc_ex(memhandler, sizeof(uint16_t), dd_ptr->smgr_handle)) == NULL )
      {
        return SNS_DDF_ENOMEM;
      }
      *num_elems = 1;
      dd_ptr->sns_dd_als_db.accuracy = dd_ptr->sns_dd_als_db.last_mlux / SNS_DD_ALS_ACCURACY_DIV_FACTOR;
      if ( dd_ptr->sns_dd_als_db.accuracy < SNS_DD_ALS_HALF_LUX )
      {
        dd_ptr->sns_dd_als_db.accuracy = SNS_DD_ALS_HALF_LUX;
      }
      DDF_MSG_0(HIGH, "accuracy");
      //SNS_PRINTF_STRING_ID_MEDIUM_1(SNS_DBG_MOD_DSPS_DD_ALSPRX,
                                    //DBG_DD_ALSPRX_GET_ATTRIB_ACC,
                                    //dd_ptr->sns_dd_als_db.accuracy);
      *accu_ptr = dd_ptr->sns_dd_als_db.accuracy;
      *(uint16_t **)value = accu_ptr;
      break;

    case SNS_DDF_ATTRIB_ODR:
      if ( (odr_ptr = sns_ddf_memhandler_malloc_ex(memhandler, sizeof(uint32_t), dd_ptr->smgr_handle)) == NULL )
      {
        return SNS_DDF_ENOMEM;
      }
      *num_elems = 1;
      *odr_ptr   = dd_ptr->chip.setup.als.odr;
      *(uint32_t **)value = odr_ptr;
      DDF_MSG_1(HIGH, "AL3320 get_attrib als odr: %d",  *odr_ptr);
      break;

    case SNS_DDF_ATTRIB_SUPPORTED_ODR_LIST:
    {
      uint32_t *odr_list;

      if((odr_list = (uint32_t *)sns_ddf_memhandler_malloc_ex(
              memhandler,
              1 * sizeof(uint32_t) , dd_ptr->smgr_handle)) == NULL)
      {
        return SNS_DDF_ENOMEM;
      }
      odr_list[0] = SNS_DD_ALSPRX_ALS_FREQ;
      *value = (uint32_t *)odr_list;
      *num_elems = 1;
    }
    break;

    default:
      //SNS_PRINTF_STRING_ID_HIGH_1(SNS_DBG_MOD_DSPS_DD_ALSPRX,
                                  //DBG_DD_ALSPRX_GET_ATTRIB_ERR,
                                  //attrib);
      return SNS_DDF_EINVALID_ATTR;
  }

  DDF_MSG_0(HIGH, "sns_dd_als_al3320_query end");
  return SNS_DDF_SUCCESS;
}

/*===========================================================================
FUNCTION      sns_dd_als_al3320_get_attrib

DESCRIPTION   sensor get attrib
===========================================================================*/
sns_ddf_status_e sns_dd_als_al3320_get_attrib
(
  sns_ddf_handle_t      handle,
  sns_ddf_sensor_e      sensor,
  sns_ddf_attribute_e   attrib,
  sns_ddf_memhandler_s* memhandler,
  void**                value,
  uint32_t*             num_elems
)
{
  sns_dd_als_state_t* state = (sns_dd_als_state_t *)handle;
  sns_ddf_status_e       status = SNS_DDF_SUCCESS;
  BOOLEAN                generic_attrib = FALSE;
  sns_ddf_registry_group_s  *reg_group;
  uint8_t                   *reg_group_data;
  sns_ddf_driver_info_s *driver_info_ptr;
  sns_ddf_driver_info_s  driver_info_l = {
    "AL3320",  /* name */
    1           /* version */
  };

  sns_ddf_device_info_s *device_info_ptr;
  sns_ddf_device_info_s  device_info_l = {
    "ALS",   /* name */
    "DynaIamge",   /* vendor */
    "AL3320",  /* model */
    1           /* version */
  };

  DDF_MSG_1(HIGH, "al3320_get_attrib() start, sensor %d", sensor);

  if ( (handle == NULL)      ||
       (memhandler == NULL) )
  {
    DDF_MSG_0(HIGH, "al3320_get_attrib() handle fail");
    return SNS_DDF_EINVALID_PARAM;
  }

  //SNS_PRINTF_STRING_ID_MEDIUM_2(SNS_DBG_MOD_DSPS_DD_ALSPRX, DBG_DD_ALSPRX_GET_ATTRIB_REQ, sensor,attrib );

  /* check first if the query is for generic attributes */
  switch ( attrib )
  {
    case SNS_DDF_ATTRIB_DRIVER_INFO:
      if ( (driver_info_ptr = sns_ddf_memhandler_malloc_ex(memhandler, sizeof(sns_ddf_driver_info_s), state->smgr_handle)) == NULL )
      {
        return SNS_DDF_ENOMEM;
      }
      *driver_info_ptr = driver_info_l; /* Is memcopy needed because of strings */
      *(sns_ddf_driver_info_s**)value = driver_info_ptr;
      *num_elems = 1;
      generic_attrib = TRUE;
      break;

    case SNS_DDF_ATTRIB_DEVICE_INFO:
      if ( (device_info_ptr = sns_ddf_memhandler_malloc_ex(memhandler, sizeof(sns_ddf_device_info_s), state->smgr_handle)) == NULL )
      {
        return SNS_DDF_ENOMEM;
      }
      *device_info_ptr = device_info_l;
      *(sns_ddf_device_info_s**)value = device_info_ptr; /* Is memcopy needed instead? */
      *num_elems = 1;
      generic_attrib = TRUE;
      break;

          
    case SNS_DDF_ATTRIB_REGISTRY_GROUP:
      DDF_MSG_0(HIGH, "in attrib reg group");
      reg_group = sns_ddf_memhandler_malloc_ex(memhandler, sizeof(*reg_group), state->smgr_handle);
      if (reg_group == NULL)
      {      
        DDF_MSG_0(HIGH, "no reg group");
        return SNS_DDF_ENOMEM;
      }

      if (state->nv_db_size == 0) 
      {
        reg_group->group_data = NULL;
        reg_group->size = 0;
        DDF_MSG_0(HIGH, "nv db size 0");
      } 
      else 
      {
        reg_group_data = sns_ddf_memhandler_malloc_ex(memhandler, state->nv_db_size, state->smgr_handle);
        if (reg_group_data == NULL)
        {
          return SNS_DDF_ENOMEM;
        }
        DDF_MSG_0(HIGH, "at memcopy");
     
        //SNS_OS_MEMCOPY(reg_group_data, &state->sns_dd_als_common_db.nv_db, sizeof(sns_dd_nv_db_type));
        memcpy(reg_group_data, &state->sns_dd_als_common_db.nv_db, sizeof(sns_dd_nv_db_type));
        reg_group->group_data = reg_group_data;
        reg_group->size = state->nv_db_size;
        DDF_MSG_0(HIGH, "after memcopy");
      }

      *value = reg_group;
      *num_elems = 1;
      generic_attrib = true;
      DDF_MSG_0(HIGH, "out of attrib group");
      return SNS_DDF_SUCCESS;

    default:
      /* do nothing */
    break;
  }

  /* The query is not for generic attribute but is for specific data type */
  if ( generic_attrib != TRUE )
  {
    switch ( sensor )
    {
      case SNS_DDF_SENSOR_AMBIENT:
        status = sns_dd_als_al3320_query(state, memhandler, attrib, value, num_elems);
        break;

      default:
        //SNS_PRINTF_STRING_ID_HIGH_1(SNS_DBG_MOD_DSPS_DD_ALSPRX,
                                    //DBG_DD_ALSPRX_GET_ATTRIB_SENSOR_TYPE_ERR,
                                    //sensor);
        status = SNS_DDF_EINVALID_ATTR;
    }
  }

  DDF_MSG_0(HIGH, "al3320_get_attrib() end");

  return status;
}

/*===========================================================================
FUNCTION      sns_dd_als_al3320_common_init

DESCRIPTION   initializes als and ps
===========================================================================*/
void sns_dd_als_al3320_common_init ( sns_dd_als_state_t* dd_ptr )
{
  dd_ptr->dropped_logs = 0;
  dd_ptr->sns_dd_als_common_db.state = SNS_DD_DEV_STOPPED;

  if (dd_ptr->sns_dd_als_common_db.nv_source == SNS_DD_ALSPRX_NV_SOURCE_DEFAULT)
  {
    //SNS_PRINTF_STRING_ID_LOW_0(SNS_DBG_MOD_DSPS_DD_ALSPRX,
                               //DBG_DD_ALSPRX_NV_USE_DRIVER_DEFAULTS);
  }
}

/*===========================================================================
FUNCTION      sns_dd_als_al3320_init_data_struct

DESCRIPTION   initialize data structures for ALS and PRX
===========================================================================*/
void sns_dd_als_al3320_init_data_struct
(
  sns_dd_als_state_t* dd_ptr
)
{
  /* Initialize the common database */
  sns_dd_als_al3320_common_init(dd_ptr);
}

/*===========================================================================
FUNCTION      sns_dd_als_al3320_set_powerstate

DESCRIPTION   set power state
===========================================================================*/
sns_ddf_status_e sns_dd_als_al3320_set_powerstate
(
  sns_dd_als_state_t* state,
  sns_ddf_powerstate_e   op_mode,
  sns_ddf_sensor_e       sensor,
  BOOLEAN                init_data
)
{
  uint8_t          i2c_data;
  sns_ddf_status_e status = SNS_DDF_SUCCESS;

  DDF_MSG_2(MED, "AL3320 var1: %d op_mode %d", 1126, op_mode);
  if ( op_mode == SNS_DDF_POWERSTATE_ACTIVE )
  {
      if(sensor == SNS_DDF_SENSOR_AMBIENT)
      {
          DDF_MSG_1(MED, "ONLY ALS", 1126);
          state->chip.setup.reg.enable |= AL3320x_ENABLE_MSK_AEN;
      }
      else if(sensor == SNS_DDF_SENSOR__ALL)
      {
          state->chip.setup.reg.enable |= AL3320x_ENABLE_MSK_AEN;
          DDF_MSG_1(MED, "AL3320 var1: %d", 5555);
      }

      i2c_data = state->chip.setup.reg.enable;
      status   = sns_dd_als_al3320_write_byte(state->port_handle, AL3320_MODE_COMMAND, &i2c_data);
      if ( status != SNS_DDF_SUCCESS ) {
          DDF_MSG_1(MED, "AL3320 var1: %d", 1127);
          return status;
      }
      DDF_MSG_2(MED, "AL3320 var1: %d var2: %d", 1128, i2c_data);
  }
  else /* when op_mode is SNS_DDF_POWERSTATE_LOWPOWER */
  {
    if (init_data == TRUE)
    {
      /* cleanup */
      sns_dd_als_al3320_init_data_struct(state);
    }

    i2c_data = SNS_DD_ALS_PWR_DOWN;
    status   = sns_dd_als_al3320_write_byte(state->port_handle, AL3320_MODE_COMMAND, &i2c_data);
    if ( status != SNS_DDF_SUCCESS )
    {
      return status;
    }
    DDF_MSG_2(MED, "AL3320 var1: %d var2: %d", 1130, i2c_data);

    state->chip.setup.prox.odr = 0;
    state->chip.setup.als.odr  = 0;
  }

  state->sns_dd_als_common_db.pwr_mode = op_mode;
  state->sns_dd_als_common_db.state    = SNS_DD_DEV_CONFIGURED;
  return status;
}

/*===========================================================================
FUNCTION      sns_dd_als_al3320_update_registers

DESCRIPTION   update registers
===========================================================================*/
sns_ddf_status_e sns_dd_als_al3320_update_registers(sns_dd_als_state_t* state)
{
    uint8_t byteVal;
    uword_u wordVal;

    lsc_setup*       setup;
    sns_ddf_status_e status = SNS_DDF_SUCCESS;
    setup  = &state->chip.setup;

    DDF_MSG_0(MED, "al3320_update_registers() start");

    /* Reset the device sns_dd_als_al3320_set_default_reg_values */ 

    /* Power off */
    byteVal = SNS_DD_ALS_PWR_DOWN;
    status = sns_dd_als_al3320_write_byte(state->port_handle, AL3320_MODE_COMMAND, &byteVal);
    if (status != SNS_DDF_SUCCESS) {
        DDF_MSG_1(MED, "al3320_update_registers() write AL3320_MODE_COMMAND err: %d", status);
        return status;
    }

    /* Flag Status */
    byteVal = SNS_CLR_ALL_INT;
    status = sns_dd_als_al3320_write_byte(state->port_handle, AL3320_FLAG_STATUS, &byteVal);
    if (status != SNS_DDF_SUCCESS) {
        DDF_MSG_1(MED, "al3320_update_registers() write AL3320_FLAG_STATUS err: %d", status);
        return status;
    }

    /* Interrupt Control */
    if (state->chip.int_gpio == INVAILD_GPIO) {
        /* polling mode disable INT */
        byteVal = SNS_DD_ALS_INT_CTRL_POLL;
    } else {
        /* DRI mode */
        byteVal = SNS_DD_ALS_INT_CTRL_DRI;
    }
    status = sns_dd_als_al3320_write_byte(state->port_handle, AL3320_INT_CTRL, &byteVal);
    if (status != SNS_DDF_SUCCESS) {
        DDF_MSG_1(MED, "al3320_update_registers() write SNS_DD_ALSPRX_INT_CTRL err: %d", status);
        return status;
    }

    /* Set ALS Threshold */
    setup->als.thresh_low = SNS_ALS_THR_LOW_F;
    wordVal.word = setup->als.thresh_low;
    status = sns_dd_als_al3320_write_word(state->port_handle, AL3320_ALS_LTHL, &wordVal);
    if ( status != SNS_DDF_SUCCESS ) {
        DDF_MSG_1(MED, "al3320_update_registers() write AL3320_ALS_LTHL err: %d", status);
        return status;
    }
    setup->als.thresh_high = SNS_ALS_THR_HIG_F;
    wordVal.word = setup->als.thresh_high;
    status = sns_dd_als_al3320_write_word(state->port_handle, AL3320_ALS_HTHL, &wordVal);
    if (status != SNS_DDF_SUCCESS) {
        DDF_MSG_1(MED, "al3320_update_registers() write AL3320_ALS_HTHL err: %d", status);
        return status;
    }

    /* ALS gain */
    setup->als.gain = AL3320_ALS_GAIN_02;
    byteVal = setup->als.gain << AL3320_ALS_GAIN_SHIFT;
    status = sns_dd_als_al3320_write_byte(state->port_handle, AL3320_ALS_CON, &byteVal);
    if (status != SNS_DDF_SUCCESS) {
        DDF_MSG_1(MED, "al3320_update_registers() write AL3320_ALS_CON err: %d", status);
        return status;
    }

    /* ALS persis */
    byteVal = AL3320_ALS_PERSIS_VAL;
    status = sns_dd_als_al3320_write_byte(state->port_handle, AL3320_ALS_PERSIS, &byteVal);
    if (status != SNS_DDF_SUCCESS) {
        DDF_MSG_1(MED, "al3320_update_registers() write AL3320_ALS_PERSIS err: %d", status);
        return status;
    }

    DDF_MSG_0(MED, "al3320_update_registers() end");

    return status;
}

/*===========================================================================
FUNCTION      sns_dd_als_al3320_reset

DESCRIPTION   reset als and ps sensor
===========================================================================*/
sns_ddf_status_e sns_dd_als_al3320_reset(sns_ddf_handle_t dd_handle)
{
    //sns_dd_als_state_t* state = (sns_dd_als_state_t *) dd_handle;
    sns_ddf_status_e       status = SNS_DDF_SUCCESS;

    DDF_MSG_1(MED, "al3320_reset() AL3320 var1: %d", 1149);
    
    DDF_MSG_1(MED, "al3320_reset() AL3320 reset exit: %d", status);
    return status;
}

/*===========================================================================
FUNCTION      sns_dd_als_al3320_set_attrib

DESCRIPTION   set attrib
===========================================================================*/
sns_ddf_status_e sns_dd_als_al3320_set_attrib
(
  sns_ddf_handle_t     dd_handle,
  sns_ddf_sensor_e     sensor,
  sns_ddf_attribute_e  attrib,
  void*                value
)
{
    sns_dd_als_state_t *state  = dd_handle;
    sns_ddf_status_e       status = SNS_DDF_SUCCESS;

    DDF_MSG_2(MED, "AL3320 set attrib var1: %d attrib %d", 1153, attrib);

  if ( (dd_handle == NULL) ||
       (value == NULL) )
  {
    return SNS_DDF_EINVALID_PARAM;
  }

  switch ( attrib )
  {
  case SNS_DDF_ATTRIB_POWER_STATE:

      /* set power mode */
      DDF_MSG_1(MED, "power mode set %d", 1154);
      status = sns_dd_als_al3320_set_powerstate((sns_dd_als_state_t *)dd_handle,
                                            *(sns_ddf_powerstate_e *)value,
                                            sensor,
                                            TRUE);
      if(status != SNS_DDF_SUCCESS)
      {
          DDF_MSG_1(MED, "AL3320 var1: %d", 1154);
          return status;
      }

      break;

  case SNS_DDF_ATTRIB_ODR:
          if(sensor == SNS_DDF_SENSOR_AMBIENT)
          {
             state->chip.setup.als.odr = *(uint32_t*)value;
          }
          else
          {
             return SNS_DDF_EINVALID_ATTR;
          }
      break;

  default:
      return SNS_DDF_EINVALID_PARAM;
  }        
        if(state->chip.setup.als.odr > SNS_ALSPRO_INT_ODR) {
          state->chip.setup.als.odr = SNS_ALSPRO_INT_ODR;
          DDF_MSG_1(MED, "al3320_set_attrib() setup.als.odr = %d", state->chip.setup.als.odr);
        }

  return(status);
}

/*===========================================================================
FUNCTION      sns_dd_als_al3320_run_test

DESCRIPTION   sensor self test
===========================================================================*/
sns_ddf_status_e sns_dd_als_al3320_run_test
( sns_ddf_handle_t  dd_handle,
  sns_ddf_sensor_e  sensor,
  sns_ddf_test_e    test,
  uint32_t*         err
)
{
    sns_dd_als_state_t *state  = dd_handle;
    sns_ddf_status_e       status = SNS_DDF_SUCCESS;
    sns_ddf_handle_t       smgr_handle;
    uint8_t readByte;

    DDF_MSG_2(HIGH, "run test: sensor %d test %d", sensor, test);

    if(sensor == SNS_DDF_SENSOR_AMBIENT)
    {
        DDF_MSG_0(HIGH, "in selftest sensor_ambient ");
        if(test == SNS_DDF_TEST_SELF)
        {
                status = sns_dd_als_al3320_read_byte(state->port_handle, AL3320_ALS_CON, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;
                DDF_MSG_1(HIGH, "sns_dd_als_al3320_run_test() read AL3320_ALS_CON value = 0x%02x",readByte);

                status = sns_dd_als_al3320_read_byte(state->port_handle, AL3320_ALS_PERSIS, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;
                DDF_MSG_1(HIGH, "sns_dd_als_al3320_run_test() read AL3320_ALS_PERSIS value = 0x%02x",readByte);
            
                status = sns_dd_als_al3320_read_byte(state->port_handle, AL3320_ALS_TIME, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;
                DDF_MSG_1(HIGH, "sns_dd_als_al3320_run_test() read AL3320_ALS_TIME value = 0x%02x",readByte);
            
                status = sns_dd_als_al3320_read_byte(state->port_handle, AL3320_ALS_LTHL, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;
                DDF_MSG_1(HIGH, "sns_dd_als_al3320_run_test() read AL3320_ALS_LTHL value = 0x%02x",readByte);
            
                status = sns_dd_als_al3320_read_byte(state->port_handle, AL3320_ALS_LTHH, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;
                DDF_MSG_1(HIGH, "sns_dd_als_al3320_run_test() read AL3320_ALS_LTHH value = 0x%02x",readByte);
                
                status = sns_dd_als_al3320_read_byte(state->port_handle, AL3320_ALS_HTHL, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;
                DDF_MSG_1(HIGH, "sns_dd_als_al3320_run_test() read AL3320_ALS_HTHL value = 0x%02x",readByte);
                
                status = sns_dd_als_al3320_read_byte(state->port_handle, AL3320_ALS_HTHH, &readByte);
            if( status != SNS_DDF_SUCCESS ) 
                return status;
                DDF_MSG_1(HIGH, "sns_dd_als_al3320_run_test() read AL3320_ALS_HTHH value = 0x%02x",readByte);
        } else if(test == SNS_DDF_TEST_OEM) {
            DDF_MSG_0(HIGH, "in selftest sensor_ambient OEM test ");
            smgr_handle = state->smgr_handle;

            status = sns_dd_als_al3320_reset(state);

            if( status != SNS_DDF_SUCCESS )
            {
                DDF_MSG_0(HIGH, "failed in reset ");
                    sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_AMBIENT, SNS_DDF_EFAIL, SNS_DD_AL3320_ALS_CAL_FAILED_RST);
                return status;
            }
            DDF_MSG_0(HIGH, "after reset"); 

            /*  enable sensor */
                status = sns_dd_als_al3320_set_powerstate(dd_handle, SNS_DDF_POWERSTATE_ACTIVE ,SNS_DDF_SENSOR__ALL, TRUE);
            if( status != SNS_DDF_SUCCESS )
            {
                DDF_MSG_0(HIGH, "failed in set power state -sensor enable ");
                    sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_AMBIENT, SNS_DDF_EFAIL, SNS_DD_AL3320_ALS_CAL_FAILED_ENB);
                return status;
            }

            DDF_MSG_0(HIGH, "out of als scaling");
            status = sns_ddf_smgr_notify_event(smgr_handle, SNS_DDF_SENSOR_AMBIENT, SNS_DDF_EVENT_UPDATE_REGISTRY_GROUP);
            if( status != SNS_DDF_SUCCESS )
            {
                DDF_MSG_0(HIGH, "failed in selftest -no registry notify");
                    sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_AMBIENT, SNS_DDF_EFAIL, SNS_DD_AL3320_ALS_CAL_FAILED_REG);
                return status;
            }
            DDF_MSG_0(HIGH, "successful registry update notify complete");
        } else {
            DDF_MSG_0(HIGH, "NOT OEM test ");
            return SNS_DDF_EINVALID_TEST;
        }
    } else {
        DDF_MSG_0(HIGH, "NOT CORRECT Sensor");
        return SNS_DDF_EINVALID_TEST;
    }

    return SNS_DDF_SUCCESS;
}

/*===========================================================================
FUNCTION      sns_dd_als_al3320_probe

DESCRIPTION   sensor probe
===========================================================================*/
sns_ddf_status_e sns_dd_als_al3320_probe
(
 sns_ddf_device_access_s* device_info,
 sns_ddf_memhandler_s*    memhandler,
 uint32_t*                num_sensors,
 sns_ddf_sensor_e**       sensors
)
{
  sns_ddf_status_e status;
  sns_ddf_handle_t port_handle;
  uint8_t i2c_buff;
  uint8_t bytes_read;

  *num_sensors = 0;
  *sensors = NULL;

  status = sns_ddf_open_port(&port_handle, &(device_info->port_config));
  if(status != SNS_DDF_SUCCESS)
  {
    return status;
  }

  /* Read & Verify Device ID */
  status = sns_ddf_read_port( port_handle,
                              AL3320_MODE_COMMAND,
                              &i2c_buff,
                              1,
                              &bytes_read );
                             
  if(status != SNS_DDF_SUCCESS)
  {
    sns_ddf_close_port( port_handle );
    return status;
  }

  *num_sensors = SNS_DD_ALSPRX_NUM_SENSOR_TYPES;
  *sensors = sns_ddf_memhandler_malloc( memhandler,
                                        sizeof(sns_ddf_sensor_e) * *num_sensors );
  if( *sensors != NULL )
  {    
    (*sensors)[0] = SNS_DDF_SENSOR__NONE;
    (*sensors)[1] = SNS_DDF_SENSOR_AMBIENT; 
    status = SNS_DDF_SUCCESS;
  } else {
    status = SNS_DDF_ENOMEM;
  }
    sns_ddf_close_port( port_handle );
  return status;
}

